import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

Settings = class();

function Settings:Constructor()
    self:_Load();
end

-- encode numbers as strings if necessary for the client to save properly
function Settings:EncodeNumbers(object)
    if ("en" == Turbine.Engine.GetLocale()) then
        return object;
    end

    local lookup_table = {};
    local function _encode(object)
        if (type(object) == "number") then
            return ("<num>%f</num>"):format(object);
        elseif (type(object) ~= "table") then
            return object;
        elseif (lookup_table[object]) then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
            new_table[_encode(index)] = _encode(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _encode(object);
end

-- decode numbers encoded as strings
function Settings:DecodeNumbers(object)
    if ("en" == Turbine.Engine.GetLocale()) then
        return object;
    end
    
    local lookup_table = {};
    local function _decode(object)
        if (type(object) == "string") then
            local match = object:match("<num>(%d+.%d+)</num>");
            if (match ~= nil) then
                return tonumber(match);
            else
                return object;
            end
        elseif (type(object) ~= "table") then
            return object;
        elseif lookup_table[object] then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
            new_table[_decode(index)] = _decode(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _decode(object);
end

function Settings:_Load()
    local load_function = Turbine.PluginData.Load; -- aliased prevent the EU fix from changing this file
    local saved = load_function(Turbine.DataScope.Character, "BuffBarsSettings");

    if (type(saved) ~= "table") then
        -- set some reasonable defaults
        self:SetDefaults();
        self:Save();
    else
        -- convert the data format back for non US clients if necessary
        saved = self:DecodeNumbers(saved);

        -- copy the values into this object
        for i,v in pairs(saved) do
            if (i == "windows") then
                self.windows = {};
                local size = table.getn(v);
                for i = 1, size do
                    self.windows[i] = EffectBarSettings(v[i]);
                end
            elseif (i == "quickslots") then
                self.quickslots = {};
                local size = table.getn(v);
                for i = 1, size do
                    self.quickslots[i] = QuickslotSettings(v[i]);
                end
            elseif (i == "sliders") then
                self.sliders = {};
                local size = table.getn(v);
                for i = 1, size do
                    self.sliders[i] = SliderSettings(v[i]);
                end
            else
                self[i] = v;
            end
        end
        
        -- see if we need to do a migration
        self:CheckMigration();
    end
end

function Settings:_Save()
    -- convert the data format to a safe one for non US clients if necessary
    local object = self:EncodeNumbers(self);
    local save_function = Turbine.PluginData.Save; -- aliased prevent the EU fix from changing this file
    save_function(Turbine.DataScope.Character, "BuffBarsSettings", object);
end

function Settings:Migrate(version)
    Turbine.Shell.WriteLine("Migrating BuffBars options to version " .. version);
    self.version = version;
end

function Settings:CheckMigration()
    local save = false;
    if (self.version == 1) then
        self.fillBottomToTop = false;
        self.showEffectName = true;
        self.showEffectTimer = true;
        self.timerWidth = nil;
        self:Migrate(2);
        save = true;
    end
    if (self.version == 2) then
        self.quickslots = {QuickslotSettings()};
        self.quickslots[1].enabled = self.showQuickslot;
        self.showQuickslot = nil;
        self.quickslots[1].qsX = self.qsX;
        self.qsX = nil;
        self.quickslots[1].qsY = self.qsY;
        self.qsY = nil;
        self.quickslots[1].moralePercent = self.moralePercent;
        self.moralePercent = nil;
        self.quickslots[1].powerPercent = self.powerPercent;
        self.powerPercent = nil;
        self.quickslots[1].moraleType = self.moraleType;
	    self.moraleType = nil;
        self.quickslots[1].moraleData = self.moraleData;
	    self.moraleData = nil;
        self.quickslots[1].powerType = self.powerType;
	    self.powerType = nil;
        self.quickslots[1].powerData = self.powerData;
	    self.powerData = nil;
        self.quickslots[1].diseaseType = self.diseaseType;
	    self.diseaseType = nil;
        self.quickslots[1].diseaseData = self.diseaseData;
	    self.diseaseData = nil;
        self.quickslots[1].fearType = self.fearType;
	    self.fearType = nil;
        self.quickslots[1].fearData = self.fearData;
	    self.fearData = nil;
        self.quickslots[1].poisonType = self.poisonType;
        self.poisonType = nil;
        self.quickslots[1].poisonData = self.poisonData;
        self.poisonData = nil;
        self.quickslots[1].woundType = self.woundType;
        self.woundType = nil;
        self.quickslots[1].woundData = self.woundData;
        self.woundData = nil;
        
        self.windows = {EffectBarSettings()};
        self.windows[1].posX = self.posX;
    	self.posX = nil;
        self.windows[1].posY = self.posY;
    	self.posY = nil;
        self.windows[1].barHeight = self.barHeight;
    	self.barHeight = nil;
        self.windows[1].barWidth = self.barWidth;
    	self.barWidth = nil;
        self.windows[1].showDebuffBar = self.showDebuffBar;
    	self.showDebuffBar = nil;
        self.windows[1].showBuffBar = self.showBuffBar;
    	self.showBuffBar = nil;
        self.windows[1].showDisease = self.showDisease;
    	self.showDisease = nil;
        self.windows[1].showFear = self.showFear;
    	self.showFear = nil;
        self.windows[1].showPoison = self.showPoison;
    	self.showPoison = nil;
        self.windows[1].showWound = self.showWound;
    	self.showWound = nil;
        self.windows[1].showCorruption = self.showCorruption;
    	self.showCorruption = nil;
        self.windows[1].showCry = self.showCry;
    	self.showCry = nil;
        self.windows[1].showElemental = self.showElemental;
    	self.showElemental = nil;
        self.windows[1].showPhysical = self.showPhysical;
    	self.showPhysical = nil;
        self.windows[1].showSong = self.showSong;
    	self.showSong = nil;
        self.windows[1].showUndefined = self.showUndefined;
    	self.showUndefined = nil;
        self.windows[1].showCurableOnly = self.showCurableOnly;
    	self.showCurableOnly = nil;
        self.windows[1].ignored = self.ignored;
    	self.ignored = nil;
        self.windows[1].showEffectName = self.showEffectName;
    	self.showEffectName = nil;
        self.windows[1].showEffectTimer = self.showEffectTimer;
    	self.showEffectTimer = nil;
        self.windows[1].backgroundColor = self.backgroundColor;
    	self.backgroundColor = nil;
        self.windows[1].font = self.font;
    	self.font = nil;
        self.windows[1].sortMethod = self.sortMethod;
    	self.sortMethod = nil;
        self.windows[1].sortAscending = self.sortAscending;
    	self.sortAscending = nil;
        self.windows[1].showDebuffsFirst = self.showDebuffsFirst;
    	self.showDebuffsFirst = nil;
        self.windows[1].fillBottomToTop = self.fillBottomToTop;
    	self.fillBottomToTop = nil;
        self.windows[1].allowFlashing = self.allowFlashing;
    	self.allowFlashing = nil;
        self.windows[1].flashTime = self.flashTime;
    	self.flashTime = nil;

        local col = Colors.Flash;
        self.windows[1].colors[col].remaining = self.flashRemainingColor;
        self.windows[1].colors[col].elapsed = self.flashElapsedColor;
        self.flashRemainingColor = nil;
        self.flashElapsedColor = nil;
        
        col = Colors.Undefined;
        self.windows[1].colors[col].remaining = self.undefinedRemainingColor;
        self.windows[1].colors[col].elapsed = self.undefinedElapsedColor;
        self.undefinedRemainingColor = nil;
        self.undefinedElapsedColor = nil;
        
        col = Colors.Disease;
        self.windows[1].colors[col].remaining = self.diseaseRemainingColor;
        self.windows[1].colors[col].elapsed = self.diseaseElapsedColor;
        self.diseaseRemainingColor = nil;
        self.diseaseElapsedColor = nil;
        
        col = Colors.Fear;
        self.windows[1].colors[col].remaining = self.fearRemainingColor;
        self.windows[1].colors[col].elapsed = self.fearElapsedColor;
        self.fearRemainingColor = nil;
        self.fearElapsedColor = nil;
    
        col = Colors.Poison;
        self.windows[1].colors[col].remaining = self.poisonRemainingColor;
        self.windows[1].colors[col].elapsed = self.poisonElapsedColor;
        self.poisonRemainingColor = nil;
        self.poisonElapsedColor = nil;
    
        col = Colors.Wound;
        self.windows[1].colors[col].remaining = self.woundRemainingColor;
        self.windows[1].colors[col].elapsed = self.woundElapsedColor;
        self.woundRemainingColor = nil;
        self.woundElapsedColor = nil;
    
        col = Colors.Physical;
        self.windows[1].colors[col].remaining = self.physicalRemainingColor;
        self.windows[1].colors[col].elapsed = self.physicalElapsedColor;
        self.physicalRemainingColor = nil;
        self.physicalElapsedColor = nil;
    
        col = Colors.Elemental;
        self.windows[1].colors[col].remaining = self.elementalRemainingColor;
        self.windows[1].colors[col].elapsed = self.elementalElapsedColor;
        self.elementalRemainingColor = nil;
        self.elementalElapsedColor = nil;
    
        col = Colors.Song;
        self.windows[1].colors[col].remaining = self.songRemainingColor;
        self.windows[1].colors[col].elapsed = self.songElapsedColor;
        self.songRemainingColor = nil;
        self.songElapsedColor = nil;
    
        col = Colors.Corruption;
        self.windows[1].colors[col].remaining = self.corruptionRemainingColor;
        self.windows[1].colors[col].elapsed = self.corruptionElapsedColor;
        self.corruptionRemainingColor = nil;
        self.corruptionElapsedColor = nil;
    
        col = Colors.Cry;
        self.windows[1].colors[col].remaining = self.cryRemainingColor;
        self.windows[1].colors[col].elapsed = self.cryElapsedColor;
        self.cryRemainingColor = nil;
        self.cryElapsedColor = nil;
        
        self:Migrate(3);
        save = true;
    end
    if (self.version == 3) then
        self.log = LogSettings();
        local i;
        for i = 1, table.getn(self.windows) do
            self.windows[i].showDiseaseCD = true;
            self.windows[i].showFearCD = true;
            self.windows[i].showPoisonCD = true;
            self.windows[i].showWoundCD = true;
            self.windows[i].showMoraleCD = true;
            self.windows[i].showPowerCD = true;
            self.windows[i].showEffectDuration = true;
            self.windows[i].minDurationFilter = false;
            self.windows[i].minDuration = 5;
            self.windows[i].maxDurationFilter = false;
            self.windows[i].maxDuration = 300;
            self.windows[i].visibleType = self.windows[i].onlyInCombat and WindowVisibility.InCombat or WindowVisibility.Always;
            self.windows[i].onlyInCombat = nil;
        end
        for i = 1, table.getn(self.quickslots) do
            self.quickslots[i].visibleType = self.quickslots[i].onlyInCombat and WindowVisibility.InCombat or WindowVisibility.Always;
            self.quickslots[i].onlyInCombat = nil;
        end

        self:Migrate(4);
        save = true;
    end
    if (self.version == 4) then
        for i = 1, table.getn(self.windows) do
            if (not self.windows[i].minDurationFilter) then
                self.windows[i].minDuration = 0;
            end
            self.windows[i].minDurationFilter = nil;
            if (not self.windows[i].maxDurationFilter) then
                self.windows[i].maxDuration = 0;
            end
            self.windows[i].maxDurationFilter = nil;
            self.windows[i].displayFormat = TimeDisplayFormat.Clock;
        end
            
        self.sliders = {};
        self:Migrate(5);
        save = true;
    end
    
    if (save) then
        self:Save();
    end
end

function Settings:SetDefaults()
    -- current settings version
    self.version = 5;

    -- quickslot 
    self.quickslots = {QuickslotSettings()};
    
    -- effect windows
    self.windows = {EffectBarSettings()};
    
    -- effect sliders
    self.sliders = {SliderSettings()};
    
    -- logging
    self.log = LogSettings();
end

-- class level stuff
function Settings.Load()
    Settings.instance = Settings();
end

-- load the plugin settings
function Settings.Save()
    if (Settings.instance ~= nil) then
        Settings.instance:_Save();
    end
end

